# css-source-map-rebase

[![NPM version][npm-image]][npm-url] [![Build Status][travis-image]][travis-url] [![Coverage percentage][coveralls-image]][coveralls-url]

> Rebase your CSS assets relatively to the source file they were imported from.

## Installation

```bash
npm install css-source-map-rebase
```

## Example

Consider the following SASS sources:

index.scss

``` css
@import "partial/bar.scss";

.foo {
    background: url('./foo.png');
}
```

partial/bar.scss

``` css
.bar {
    background: url('./bar.png');
}
```

By rebasing the assets relatively to the file they were imported from, the resulting CSS would be:

``` css
.bar {
    background: url('partial/bar.png');
}

.foo {
    background: url('foo.png');
}
```

## How it works

css-source-map-rebase uses the mapping provided by a source map to resolve the original file the assets where imported from. Any tool able to generate a source map from a stylesheet source file (may it be SASS, LESS or any other pre-processor language) is appropriate. Here is how one could use node-sass and css-source-map-rebase together to render a stylesheet and rebase its assets.

``` javascript
const {render} = require('node-sass');
const {Rebaser} = require('css-source-map-rebase');

render({
    file: 'index.scss',
    sourceMap: true,
    sourceMapEmbed: sourceMapEmbed,
    outFile: 'index.css'
}, (error, result) => {
    if (error) {
        // do something on error
    }
    else {        
        let rebaser = new Rebaser();

        rebaser.rebase(result.css)
          .then(result => {
              // result.css contains the rendered stylesheet with rebased assets
              // result.map contains the source map
          });
    }
});
```

## API

Read the [documentation](https://nightlycommit.github.io/css-source-map-rebase) for more information.

## Contributing

* Fork the main repository
* Code
* Implement tests using [tape](https://www.npmjs.com/package/tape)
* Issue a pull request keeping in mind that all pull requests must reference an issue in the issue queue

### Running the test suite

The test suite can be run by executing the native `test` npm script:

`npm test`

### Checking code coverage

This project adheres to a strict 100% code coverage policy. Code coverage can be checked by executing the `cover` npm script:

`npm run cover`

### Generating the documentation

The documentation can be generated by executing the `build:doc` npm script:

`npm run build:doc`

This is usually not needed as the doc will be generated on every new release.

## License

Apache-2.0 © [Eric MORAND]()

[npm-image]: https://badge.fury.io/js/css-source-map-rebase.svg
[npm-url]: https://npmjs.org/package/css-source-map-rebase
[travis-image]: https://travis-ci.org/nightlycommit/css-source-map-rebase.svg?branch=master
[travis-url]: https://travis-ci.org/nightlycommit/css-source-map-rebase
[coveralls-image]: https://coveralls.io/repos/github/NightlyCommit/css-source-map-rebase/badge.svg
[coveralls-url]: https://coveralls.io/github/NightlyCommit/css-source-map-rebase