"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const url_1 = require("url");
const source_map_1 = require("source-map");
const convert_source_map_1 = require("convert-source-map");
const events_1 = require("events");
class Rebaser extends events_1.EventEmitter {
    constructor(options = {}) {
        super();
        this._options = options;
    }
    rebase(css) {
        const isRebasable = (url) => {
            return !path_1.isAbsolute(url.href) && (url.host === null) && ((url.hash === null) || (url.path !== null));
        };
        return new Promise((resolve, reject) => {
            try {
                let originalCss = css.toString();
                let map = this._options.map;
                let rebase = this._options.rebase;
                if (!map) {
                    let converter = convert_source_map_1.fromSource(originalCss);
                    if (converter) {
                        map = Buffer.from(converter.toJSON());
                    }
                }
                if (!map) {
                    throw new Error("A map is required, either inline or explicitly passed as an option.");
                }
                let nodesToRebase = [];
                let urlRegExp = /^url\('(.*?)'\)|^url\("(.*?)"\)|^url\((.*?)\)/;
                new source_map_1.SourceMapConsumer(map.toString()).then((consumer) => {
                    let sourceMapNode = source_map_1.SourceNode.fromStringWithSourceMap(originalCss, consumer);
                    const handleNode = (node) => {
                        if (node.children) {
                            for (let child of node.children) {
                                if (typeof child === 'string') {
                                    let match = urlRegExp.exec(child);
                                    if (match !== null) {
                                        let urlStr = match[1] || match[2] || match[3];
                                        let url = url_1.parse(urlStr);
                                        if (isRebasable(url)) {
                                            let resolvedPath = path_1.posix.join(path_1.dirname(node.source), url.pathname);
                                            const done = (rebasedPath) => {
                                                if (rebasedPath !== false) {
                                                    if (!rebasedPath) {
                                                        rebasedPath = resolvedPath;
                                                    }
                                                    nodesToRebase.push({
                                                        sourceMapNode: node,
                                                        originalPath: urlStr,
                                                        rebasedPath: rebasedPath
                                                    });
                                                }
                                            };
                                            if (!rebase) {
                                                rebase = (node, path, done) => {
                                                    done();
                                                };
                                            }
                                            rebase(node.source, resolvedPath, done);
                                        }
                                    }
                                }
                                handleNode(child);
                            }
                        }
                    };
                    handleNode(sourceMapNode);
                    for (let nodeToRebase of nodesToRebase) {
                        nodeToRebase.sourceMapNode.replaceRight(nodeToRebase.originalPath, nodeToRebase.rebasedPath);
                        this.emit("rebase", nodeToRebase.rebasedPath);
                    }
                    let codeWithSourceMap = sourceMapNode.toStringWithSourceMap();
                    resolve({
                        css: Buffer.from(codeWithSourceMap.code),
                        map: Buffer.from(codeWithSourceMap.map.toString())
                    });
                });
            }
            catch (err) {
                reject(err);
            }
        });
    }
}
exports.Rebaser = Rebaser;
